# -*- coding: utf-8 -*-

# Copyright: (c) 2019 Ansible, Inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


class ModuleDocFragment(object):

    # Standard files documentation fragment
    DOCUMENTATION = r'''
options: {}
notes:
  - This module is supported on C(ansible_network_os) network platforms. See
    the :ref:`Network Platform Options <platform_options>` for details.
'''
