#!/usr/bin/env bash

if [ "x$WILDFLY_HOME" = "x" ]; then
    WILDFLY_HOME="{{ wf_homedir }}"
fi

if [[ "$1" == "domain" ]]; then
    $WILDFLY_HOME/bin/domain.sh -c "$2" -b "$3"
else
    $WILDFLY_HOME/bin/standalone.sh -c "$2" -b "$3"
fi
