Windows Registry Editor Version 5.00

[HKEY_CURRENT_USER\{{ test_reg_path }}]

[HKEY_CURRENT_USER\{{ test_reg_path }}\nested]
"string"="test"
"binary"=hex:01,16
"dword"=dword:00000001
"qword"=hex(b):01,00,00,00,00,00,00,00
"large_dword"=dword:ffffffff
"large_qword"=hex(b):ff,ff,ff,ff,ff,ff,ff,ff
"multi"=hex(7):61,00,2c,00,20,00,62,00,00,00,63,00,00,00,00,00
"expand"=hex(2):25,00,77,00,69,00,6e,00,64,00,69,00,72,00,25,00,5c,00,64,00,69,\
  00,72,00,00,00

[HKEY_CURRENT_USER\{{ test_reg_path }}\nested\nest1]
"dontcare"=""

[HKEY_CURRENT_USER\{{ test_reg_path }}\nested\nest2]


[HKEY_CURRENT_USER\{{ test_reg_path }}\single]
"string1"=""
"string2"="abc123"
"none"=hex(0):
"none1"=hex(0):00

[HKEY_CURRENT_USER\{{ test_reg_path }}\Empty Default]


[HKEY_CURRENT_USER\{{ test_reg_path }}\Blank Default]
@=""
"(Default)"=""

[HKEY_CURRENT_USER\{{ test_reg_path }}\Duplicate Default]
@="default"
"(Default)"="custom"
