plugin: aws_rds
aws_access_key_id: '{{ aws_access_key }}'
aws_secret_access_key: '{{ aws_secret_key }}'
{% if security_token | default(false) %}
aws_security_token: '{{ security_token }}'
{% endif %}
regions:
  - '{{ aws_region }}'
keyed_groups:
  - key: 'db_parameter_groups|json_query("[].db_parameter_group_name")'
    prefix: rds_parameter_group
  - key: tags
    prefix: tag
  - key: engine
    prefix: rds
filters:
  db-instance-id: "{{ resource_prefix }}-mariadb"
